#This file is part of Tryton.  The COPYRIGHT file at the top level of
#this repository contains the full copyright notices and license terms.
import random
from trytond.model import fields
from trytond.pool import PoolMeta

# Size photo default
WIDTH = 200
HEIGHT = 250

__all__ = ['Employee']
__metaclass__ = PoolMeta


class Employee:
    __name__ = 'company.employee'
    id_pass = fields.Char('Id. Pass', size=None, readonly=True, 
            select=True)
    base_salary = fields.Property(fields.Numeric('Base Salary', 
            digits=(16, 2),
            help='Base salary for apply to Social Security Laws'))
    salary_day = fields.Function(fields.Numeric('Salary Day',
            digits=(16, 2)), 'get_salary_day')
    photo = fields.Binary('Photo')
    department = fields.Many2One('company.department', 'Department', 
            required=False)
    blood_group = fields.Selection([
            ('', ''),
            ('rh_op', 'Rh O+'),
            ('rh_on', 'Rh O-'),
            ('rh_ap', 'Rh A+'),
            ('rh_an', 'Rh A-'),
            ('rh_bp', 'Rh B+'),
            ('rh_bn', 'Rh B-'),
            ('rh_abp', 'Rh AB+'),
            ('rh_abn', 'Rh AB-'),
            ], 'Blood Group', select=True)

    @classmethod
    def __setup__(cls):
        super(Employee, cls).__setup__()

    def get_salary_day(self, name=None):
        if self.salary:
            return (self.salary / 30)

    @classmethod
    def create(cls, vals):
        new_vals = []
        for val in vals:
            val['id_pass'] = cls._get_password()
            new_vals.append(val)
        return super(Employee, cls).create(new_vals)

    """
    TODO: Add wizard resize photo
        :: vals['photo'] = cls._resize_photo(self.photo)
        :: super(Employee, cls).write(employees, vals)
    """

    @classmethod
    def _get_password(cls):
        list_passwords = [employee.id_pass for employee in cls.search([])]
        invalid_id_pass = True
        while invalid_id_pass:
            id_pass = str(random.randrange(1001, 9999))
            if id_pass not in list_passwords:
                invalid_id_pass = False
        return id_pass

    @classmethod
    def _resize_photo(cls, img_buff):
        import Image
        from StringIO import StringIO

        io_image = StringIO(img_buff)

        if io_image.len == 0:
            return None

        img = Image.open(io_image)
        width, height = img.size
        
        # To try center original photo for resize it
        factor_resize = float(WIDTH)/width
        height_new = int(height * factor_resize)
        
        # To resize photo to standar size (WIDTH, HEIGHT)
        img_resized = img.resize((WIDTH, height_new),)
        img_resized = img_resized.crop((0, 0, WIDTH, HEIGHT))
        #img_resized.show()
        
        io_obj = StringIO()
        img_resized.save(io_obj, 'JPEG')
        img_data_buff = io_obj.getvalue()
        io_obj.close()
        return img_data_buff
